#include <errno.h>
#include <time.h>

#ifndef NONAMESPACE

#include <iostream>
using namespace std;

#else

#include <iostream.h>

#endif

#include <string>

#include "pidfunc.h"

#ifndef strptime
#include "strptime.h"
#endif

#define DATEFORMAT	"%Y/%m/%d %H:%M:%S"

int str2time( string str, bool quiet, time_t *pts )
{
	tm ttm;

	if( !strptime( str.c_str(), DATEFORMAT, &ttm ) )
	{
	    /*
	     * Date string is not in the expected format.
	     */

	    if( !quiet )
	    {
		/*
		 * Caller wants us to complain about errors.
		 */
		cout << "strptime(): Invalid date string, expecting ";
		cout << DATEFORMAT << "\n";
	    }

	    /*
	     * Return value indicating that an invalid format was encountered.
	     */
	    return EINVAL;
	}

	ttm.tm_isdst = -1;	// don't care about Daylight Savings Time

	*pts = mktime( &ttm );

	return 0;
}

int str2time( string str, time_t *pts )
{
	/*
	 * Complain about any errors encoutered in str2time().
	 */
	return str2time( str, false, pts );
}

void time2str( time_t ts, string *str )
{
#define TIMEBUFSIZE	20

	tm *pttm;
	char buf[TIMEBUFSIZE];

	pttm = localtime( &ts );
	strftime( buf, TIMEBUFSIZE, DATEFORMAT, pttm );
	*str = buf;
}

void time2ODBCstr( time_t ts, string *str )
{
#define TIMEBUFSIZE	20

	tm *pttm;
	char buf[TIMEBUFSIZE];

	pttm = localtime( &ts );
	strftime( buf, TIMEBUFSIZE, "%Y-%m-%d %H:%M:%S", pttm );
	*str = buf;
}


int tscmp( const void *ppidfunc1, const void *ppidfunc2 )
{
	double diffts;

	diffts = difftime( ( *( PidFunc ** )ppidfunc1 )->GetStartTS(),
	    ( *( PidFunc ** )ppidfunc2 )->GetStartTS() );

	if( diffts < 0 )
	    return -1;
	else if( diffts > 0 )
	    return 1;
	else
	    return 0;
}
